@@ -89,11 +89,15 @@ tess.create.phylo <- function(times,root=FALSE,tip.label=NULL) {
 
 tess.prepare.pdf <- function(lambda, mu, massExtinctionTimes,
                          massExtinctionSurvivalProbabilities,
-                         age, t.crit.f) {
+                         age, t.crit.f,forLikelihood=FALSE) {
   
   ## Constants for now, but this is the resolution of reporting
   ## (different to calculation) for the two stages of integration.
-  n1 <- 1001
+  if ( forLikelihood == TRUE ) {
+    n1 <- 2
+  } else {
+    n1 <- 1001
+  }
 
   
   if ( length(massExtinctionTimes) == 0 ) {
@@ -133,16 +137,15 @@ tess.ode.piecewise <- function(lambda, mu, times, t.crit, t.crit.p) {
 
   tmax <- times[length(times)]
   n <- length(t.crit) - 1L
-  t.split <- r.split <- s.split <- vector("list", n)
   bin <- findInterval(times, t.crit, TRUE)
   y <- c(0)
 
   xx <- c()
   r_points <- c()
-  s_points <- c()
+  # we compute the integrals stepwise from t.crit[i] to t.crit[i+1]
   for ( i in seq_len(n) ) {
     j <- i + 1L
-    t.split[[i]] <- ti <- c(t.crit[[i]], times[bin == i], t.crit[[j]]-1E-9)
+    ti <- c(t.crit[[i]], times[bin == i], t.crit[[j]]-1E-9)
     yi <- lsoda(y, ti, obj, tcrit=t.crit[[j]])
 
     xx <- c(xx,ti)
@@ -150,83 +153,38 @@ tess.ode.piecewise <- function(lambda, mu, times, t.crit, t.crit.p) {
     
     y <- yi[nrow(yi),-1]
 
+    # is there a mass-extinction (or sampling) event at the event of the time-interval?
     if ( !is.na(t.crit.p[[j]]) ) {
       y[1] <- y[1] - log(t.crit.p[[j]])
     }
   }
-  xx <- c(xx, tmax + 1e-8)
-  r_points <- c(r_points,r_points[length(r_points)])
+
+  # add a final point slightly over the given time interval. we need this so that some of the numerical routines do not break when calling r(t).
+  xx <- c(-1E-4 ,xx, tmax + 1e-8)
+  r_points <- c(0,r_points,r_points[length(r_points)])
   rate <- approxfun(xx,r_points)
 
+  # compute the integral int_{x}^{T} mu(t)*exp(r(t)) dt
   f <- function(t) mu(t)*exp(rate(t))
   probs <- array(0,length(xx))
   for (i in length(xx):2) {
     u <- xx[i]
     l <- xx[i-1]
-    val <- integrate(f,upper=u,lower=l)$value / exp(rate(l))
-    probs[i-1] <- val + probs[i] * exp(rate(u) - rate(l))
-  }
-  p_s <- approxfun(xx,probs)
+    val <- integrate(f,upper=u,lower=l)$value
+    probs[i-1] <- val + probs[i]
 
-  list(r=rate,s=p_s)
-}
-
-## This carries out integration for the time-varying
-## speciation/extinction model with functions lambda and mu,
-## outputting at the times in the vector 'times'.  The vectors
-## 't.crit'  and 't.crit.p' contain break points or mass-extinction events.
-tess.ode.piecewise2 <- function(lambda, mu, times, t.crit, t.crit.p) {
-  obj <- function(t, state, parms) {
-    r <- mu(tmax-t) - lambda(tmax-t)
-    s <- mu(tmax-t) * exp(state[[1]])
-    list(c(r,s))
-  }
-
-  tmax <- times[length(times)]
-  n <- length(t.crit) - 1L
-  t.split <- r.split <- s.split <- vector("list", n)
-  bin <- findInterval(times, t.crit, TRUE)
-  y <- c(0,0)
-
-  xx <- c()
-  r_points <- c()
-  s_points <- c()
-  for ( i in seq_len(n) ) {
-    j <- i + 1L
-    t.split[[i]] <- ti <- c(t.crit[[i]], times[bin == i], t.crit[[j]]-1E-9)
-    yi <- lsoda(y, ti, obj, tcrit=t.crit[[j]])
-
-    xx <- c(xx,ti)
-    r_points <- c(r_points,yi[,2])
-    s_points <- c(s_points,yi[,3])
-    
-    y <- yi[nrow(yi),-1]
-
-    if ( !is.na(t.crit.p[[j]]) ) {
-      y[1] <- y[1] - log(t.crit.p[[j]])
-      y[2] <- y[2] - (t.crit.p[[j]]-1) * exp(y[1])
+    # add the mass-extinction
+    event <- which( t.crit == l )
+    if ( length(event) > 0 && !is.na(t.crit.p[[event[1]]]) ) {
+      probs[i-1] <- probs[i-1] - (t.crit.p[[event]]-1)*exp(rate(t.crit[[event]]))
     }
   }
-  xx <- c(xx, tmax + 1e-8)
-  r_points <- c(r_points,r_points[length(r_points)])
-  s_points <- c(s_points,s_points[length(s_points)])
-  rate <- approxfun(xx,r_points[length(r_points)]-rev(r_points))
+  surv <- approxfun(xx,probs)
 
-  samples <- rev(s_points)
-  p <- array(0,length(xx))
-  for (i in length(xx):2) {
-    u <- xx[i]
-    l <- xx[i-1]
-    val <- (samples[i-1] - samples[i]) / exp(rate(tmax) - rate(u))
-    p[i-1] <- val + p[i] * exp(rate(u) - rate(l))
-  }
-  p_s <- approxfun(xx,p)
-
-  list(r=rate,s=p_s)
+  list(r=rate,s=surv)
 }
 
 
-
 find.max <- function(pdf,m,n) {
   
   xx <- seq(0, m, length=n)
